/* filename:    dsge_pm.g
** description: The program maximizes the posterior density of the
**              dsge model
*/

format /mat /on /mb1 /ros 16,8;

/** Global defaults **/
__output = 1;       /** 1 for verbose of CSMIN**/
__graph  = 1;       /** 1 for graph mode **/
_verbose = 2;       /** 0 for silence, 1 for compact, 2 for detailed**/
_fcmptol = 1E-10;

/********************************************************
*  Import prior specification
********************************************************/
priorfile = prioname $+ ".txt";
load path = ^priopath prior[] = ^priorfile;
prior = reshape(prior,rows(prior)/5,5);

{prior,para_names} = psel(prior,msel);

pshape   = prior[.,1];
pmean    = prior[.,2];
pstdd    = prior[.,3];
pmask    = prior[.,4];
pfix     = prior[.,5];
pmaskinv = 1-pmask;
pshape   = pshape.*pmaskinv;

/*********************************************************
*   starting value for maximization
*********************************************************/
pmfile = postpath $+ "\\pm.dat";
if exist(pmfile);
  open fhpara = ^pmfile for read;
  para = readr(fhpara,1)';
  closeall fhpara;
else;
  para = pmsv(msel);
endif;

para = pmsv(msel);
npara = rows(para);
para  = para.*pmaskinv + pfix.*pmask;

/** parameter specification: transformation **/
_trspec = prtr(msel);
_trspec[.,1] = _trspec[.,1].*pmaskinv;

/** start log file **/
outstr = resupath $+ "\\" $+ ftos(timedt,"%*.*lf",1,0) $+ "pm.log";
output file = ^outstr reset;

/********************************************************
* Import data series
********************************************************/

nobs  = rows(YY);          /** number of observations **/
ny    = cols(YY);          /** number of series **/

/********************************************************
* Calculate posterior density at starting values
* (verbose mode only)
********************************************************/
if _verbose;
   "Model              " $lmodel;
   "Prior              " $lprior;
   "Dataset            " $dataselstr;
   "Subsample          " $subTstr;
   "===================================";
   "";
   "Prior Density at Starting Value:";; priodens(para,pmean,pstdd,pshape);
   lnpy = -objfcn(invtrans(para));
   "Posterior at Starting Value:    ";; lnpy;
   "";
   //"Press Any Key to Continue.."; k = keyw;
   if not(__output);
   "";"";
   "Optimization in progress..";
   "--------------------------";
   endif;
endif;      /** _verbose **/

/********************************************************
* Maximize the posterior density
********************************************************/

/** initial values: x0 and H0 **/
cc = 0;
x0 = invtrans(para) + cc* ( rndn(npara,1) );
H0 = eye(npara)*1E-4;

/** set optimization routine environment **/
nit  = 1000;        /** maximum number of iteration **/
crit = 1E-8;        /** tolerance **/

/** Use personal optimization procedure **/
/** translated from Chris Sims' CSMIN package **/
{fh,xh,g,H,itct,fcount,_csretcode} = csminwel(x0,H0,crit,nit,&objfcn);

/** clear screen **/
if __output;
   cls;
else;
   "";
endif;

/********************************************************
* save parameter estimates and return code
********************************************************/

/** max to model parameters **/
paraest = trans(xh);
paraest = real(paraest);
paraest = paraest.*pmaskinv + pfix.*pmask;


/** output file name **/
oparaest = postpath $+ "\\pm";
opostval = postpath $+ "\\pmval";
oretcode = postpath $+ "\\pmret";

/** make output file **/
create fhparaest = ^oparaest with ^para_names, npara, 8;
wr = writer(fhparaest,paraest');
closeall fhparaest;

create fhretcode = ^oretcode with RETCODE, 1, 2;
wr = writer(fhretcode,_csretcode);
closeall fhretcode;

/********************************************************
* Estimation Output
********************************************************/
{lnpy, retcode, obsmean, obsvar} = evaldsge(paraest,msel,YY);     
lnprio  = priodens( paraest, pmean, pstdd, pshape);


/** posterior values **/
postnames = "LNPOST" $~ "LNPY" $~ "LNPRIO";
create fhpostval = ^opostval with ^postnames, 3, 8;
wr = writer(fhpostval,(lnpy+lnprio)~lnpy~lnprio);
closeall fhparaest;

/* Compute gradient in terms of model parameters */
gradmod = g./ diag(gradp(&invtrans,paraest));

cls;

if _verbose;
   "";
   "Model              " $lmodel;
   "Prior              " $lprior;
   "Dataset            " $dataselstr;
   "Subsample          " $subTstr;
   "===================================";
   "Posterior Mode     " (lnpy+lnprio);
   "Likelihood at Mode " lnpy;
   "Prior at Mode      " lnprio;
   "";

   outmat = ("Parameter"  $~ "Estimate" $~ "Starting-V" $~ "Prior Mean" $~ "Prior Stdd" $~ "Gradient" ) $|
            ( (para_names') $~ ftocv( (paraest ~ trans(x0) ~ pmean ~ pstdd ~ gradmod),4,4) );

   $outmat;

   "";
   "Return code:        ";; _csretcode;
   "";

endif;


if lnpy == -1E6 ;
  end;
endif;

/********************************************************
* Compute Residuals
********************************************************/

resid1 = YY -obsmean;
resid1_m1 = meanc(resid1)';
resid1_m2 = meanc( (resid1 - resid1_m1)^2 )';
resid1_m3 = meanc( (resid1 - resid1_m1)^3 )';
resid1_m4 = meanc( (resid1 - resid1_m1)^4 )';

resid2 = (YY -obsmean)./sqrt(obsvar);
resid2_m1 = meanc(resid1)';
resid2_m2 = meanc( (resid2 - resid2_m1)^2 )';
resid2_m3 = meanc( (resid2 - resid2_m1)^3 )';
resid2_m4 = meanc( (resid2 - resid2_m1)^4 )';

/* Autocorrelation */

resid1_ar = zeros(1,3);
resid2_ar = zeros(1,3);

ind = 1;
do until ind > 3;
  resid1_ar[1,ind]=inv(resid1[1:(nobs-1),ind]'*resid1[1:(nobs-1),ind])
                   *(resid1[1:(nobs-1),ind])'*(resid1[2:nobs,ind]);
  resid2_ar[1,ind]=inv(resid2[1:(nobs-1),ind]'*resid2[1:(nobs-1),ind])
                   *resid2[1:(nobs-1),ind]'*resid2[2:nobs,ind];
  ind = ind+1;
endo;


"Raw Residuals";
"-------------";
"Mean:" resid1_m1;
"Stdd:" sqrt(resid1_m2);
"Skew:" resid1_m3./resid1_m2^(3/2);
"Kurt:" resid1_m4./(3*resid1_m2^2);
"Corr:" resid1_ar;
"";
"Standardized Residuals";
"----------------------";
"Mean:" resid2_m1;
"Stdd:" sqrt(resid2_m2);
"Skew:" resid2_m3./resid2_m2^(3/2);
"Kurt:" resid2_m4./(3*resid2_m2^2);
"Corr:" resid2_ar;
"";
ti=seqa(1,1,nobs);     
titlestr = "Output" $|
           "Hours"  $|
           "Consumption" ;

ystr     = "Raw Residual " $|
          "Stdd Residual";

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht   = 0.4;
_pnumht  = 0.4;
_ptek    = resupath $+ "\\pmresid.tkf";

begwind;
window(2,3,0);


ser_ind = 1;
do until ser_ind > 3;

  ylabel(ystr[1]);
  title(titlestr[ser_ind]);
  xy(ti,resid1[.,ser_ind]);

  ser_ind = ser_ind+1;
  nextwind;

endo;

ser_ind = 1;
do until ser_ind > 3;

  ylabel(ystr[2]);
  title(titlestr[ser_ind]);
  xy(ti,resid2[.,ser_ind]);

  ser_ind = ser_ind+1;
  nextwind;

endo;

endwind;

/** close log file **/
output file = ^outstr off;


